<?php

namespace AppBundle\Controller\Ajax;

use AppBundle\Entity\User;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Route;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Method;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpFoundation\JsonResponse;

/**
 * Class AjaxDeviceController.
 */
class AjaxDeviceController extends AjaxBaseController
{
    const ROUTER_VALIDATION_TABLE_COLUMNS = array(
        2 => 'name',
        3 => 'real_ip',
        4 => 'label',
    );

    const ROUTER_TABLE_COLUMNS = array(
        2 => 'name',
        3 => 'ovpn_ip',
        4 => 'label',
        5 => 'network_cnt',
        6 => 'online',
        7 => 'sync_status',
    );

    const DEVICE_TABLE_COLUMNS = array(
        2 => 'device_type',
        3 => 'name',
        4 => 'ovpn_ip',
        5 => 'label',
        6 => 'network_cnt',
        7 => 'online',
        8 => 'sync_status',
    );

    /**
     * Render Ajax page.
     *
     * @Route("ajax/device-table", name="app_ajax_devices_table")
     *
     * @param Request $request
     *
     * @return Response
     */
    public function ajaxDeviceAction(Request $request)
    {
        //draw=1&columns[0][data]=&columns[0][name]=&columns[0][searchable]=true&columns[0][orderable]=false&col][searcumns[0h][value]=&columns[0][search][regex]=false&columns[1][data]=name&columns[1][name]=&columns[1][searchable]=true&columns[1][orderable]=true&columns[1][search][value]=&columns[1][search][regex]=false&columns[2][data]=ovpn_ip&columns[2][name]=&columns[2][searchable]=true&columns[2][orderable]=true&columns[2][search][value]=&columns[2][search][regex]=false&columns[3][data]=validated&columns[3][name]=&columns[3][searchable]=true&columns[3][orderable]=true&columns[3][search][value]=&columns[3][search][regex]=false&columns[4][data]=online&columns[4][name]=&columns[4][searchable]=true&columns[4][orderable]=true&columns[4][search][value]=&columns[4][search][regex]=false&order[0][column]=0&order[0][dir]=asc&start=0&length=10&search[value]=&search[regex]=false&_=1511527758611

        //If is not ajax
        if (!$request->isXmlHttpRequest()) {
            return new Response('Bad Request', 400);
        }

        list($start, $length, $aOrder, $filter) = $this->parseData($request);

        //workaround for connected column (change request data so filter and order of disabled works)
        $filter = $this->filterDisabled($filter);
        $aOrder = $this->orderDisabled($aOrder);

        $filter['validated'] = '1';

        $routers = $this->apiDevice->getAllRouters($start, $length, $aOrder, $filter);

        if (empty($routers)) {
            $routers = array();
        }

        //workaround for connected column (online with value 2 used for disabled, after getting from db)
        $routers = $this->getDisabled($routers);

        $data = [
            'recordsTotal' => $this->apiDevice->getCountOfAllRouters(),
            'recordsFiltered' => count($routers),
            'data' => $routers,
        ];

        return new JsonResponse($data);
    }

    /**
     * Render Ajax page.
     *
     * @Route("ajax/devices-validation-table", name="app_ajax_validate_table")
     *
     * @param Request $request
     *
     * @return Response
     */
    public function ajaxActionValidateTable(Request $request)
    {
        //If is not ajax
        if (!$request->isXmlHttpRequest()) {
            return new Response('Bad Request', 400);
        }

        list($start, $length, $aOrder, $filter) = $this->parseData($request, self::ROUTER_VALIDATION_TABLE_COLUMNS);

        $filter['validated'] = '0';

        $routers = $this->apiDevice->getAllRouters($start, $length, $aOrder, $filter);

        if (empty($routers)) {
            $routers = array();
        }

        $data = [
            //"draw" =>  1,
            'recordsTotal' => $this->apiDevice->getCountOfAllDevices($filter),
            'recordsFiltered' => count($routers),
            'data' => $routers,
        ];

        return new JsonResponse($data);
    }

    /**
     * Render Device Detail Ajax page.
     *
     * @Route("ajax/device-detail/{id}", requirements={"id"="\d+"}, options={"expose"=true}, name="app_ajax_device_detail")
     *
     * @param Request $request
     *
     * @return Response
     */
    public function ajaxDeviceDetailAction(Request $request, $id)
    {
        //If is not ajax
        if (!$request->isXmlHttpRequest()) {
            return new Response('Bad Request', 400);
        }

        $device = $this->apiDevice->getDeviceById($id);
        if($device['device_type'] == "2") {
            $router = $this->apiDevice->getRouterById($id);
        }
        $deviceStats = $this->apiDevice->getDeviceStats($id);
        $deviceNetworks = $this->apiDevice->getNetworksByDeviceId($id);

        if ('2' == $device['device_type']) {
            $deviceLans = $this->apiDevice->getLans($id);

            $deviceTemplate = $this->renderView('app/ajax/deviceDetail.html.twig', [
                'device' => $device,
                'router' => $router,
                'lans' => $deviceLans,
                'stats' => $deviceStats,
                'networks' => $deviceNetworks,
            ]);
        } elseif ('1' == $device['device_type']) {
            $standa = $this->apiStandaloneVpnClients->getStandaloneVpnClientById($id);

            $standa['country'] = $this->getCountry($standa['country']);

            if (time() > $standa['expiration_date']) {
                $standa['red'] = 'color:#bb4a1e';
            } else {
                $standa['red'] = '';
            }

            $deviceTemplate = $this->renderView('app/ajax/deviceDetail.html.twig', [
                'device' => $device,
                'standa' => $standa,
                'stats' => $deviceStats,
                'networks' => $deviceNetworks,
            ]);
        }

        $data = array('html' => $deviceTemplate);

        return new JsonResponse($data);
    }

    /**
     * Render Ajax page.
     *
     * @Route("ajax/devices-validate", name="app_ajax_validate_selected")
     * @Method({"POST"})
     *
     * @param Request $request
     *
     * @return Response
     */
    public function ajaxValidateDevicesAction(Request $request)
    {
        //If is not ajax
        if (!$request->isXmlHttpRequest()) {
            return new Response('Bad Request', 400);
        }

        $this->denyAccessUnlessGranted(User::ROLE_DEVICE_ADMIN);

        $data = json_decode($request->getContent(), true);
        $request->request->replace($data);

        $deviceIds = $request->request->get('ids');

        try {
            $this->apiDevice->validateDevices($deviceIds);

            $data = array('status' => 'success', 'msg' => implode(',', $deviceIds));
        } catch (\Exception $e) {
            $data = array('status' => 'failed', 'msg' => implode(',', $deviceIds), 'errtext' => $e->getMessage());
        }

        return new JsonResponse($data);
    }

    /**
     * Render Ajax page.
     *
     * @Route("ajax/device-validate/{id}", options={"expose"=true}, name="app_ajax_validate_device")
     *
     * @param Request $request
     *
     * @return Response
     */
    public function ajaxValidateDeviceAction(Request $request, $id)
    {
        //If is not ajax
        if (!$request->isXmlHttpRequest()) {
            return new Response('Bad Request', 400);
        }

        $this->denyAccessUnlessGranted(User::ROLE_DEVICE_ADMIN);

        try {
            $device = $this->apiDevice->getDeviceById($id);

            $this->apiDevice->validateDevice($id);

            $data = array('status' => 'success', 'msg' => 'Device "'.$device['name'].'" was validated.');
        } catch (\Exception $e) {
            $data = array('status' => 'failed', 'msg' => 'Device "'.$device['name'].'" validation failed: '.$e->getMessage());
        }

        return new JsonResponse($data);
    }

    /**
     * Render Device Detail Ajax page.
     *
     * @Route("ajax/network-devices/{id}", defaults={"id": null}, options={"expose"=true}, name="app_ajax_network_devices")
     *
     * @param Request $request
     *
     * @return Response
     */
    public function ajaxDevicesForNetworkAction(Request $request, $id)
    {
        //If is not ajax
        if (!$request->isXmlHttpRequest()) {
            return new Response('Bad Request', 400);
        }

        list($start, $length, $aOrder, $filter) = $this->parseData($request, self::DEVICE_TABLE_COLUMNS);

        //workaround for connected column (change request data so filter and order of disabled works)
        $filter = $this->filterDisabled($filter);
        $aOrder = $this->orderDisabled($aOrder);

        $devices = $this->apiDevice->getDevicesByNetworkId($id, $start, $length, $aOrder, $filter);

        if (!empty($devices)) {
            foreach ($devices as &$arr) {
                $arr['id'] = $arr['device_id'];
                unset($arr['device_id']);
            }
            unset($arr);
        } else {
            $devices = array();
        }

        //workaround for connected column (online with value 2 used for disabled, after getting from db)
        $devices = $this->getDisabled($devices);

        $data = [
            //"draw" =>  1,
            'recordsTotal' => $this->apiDevice->getCountOfDevicesInNetwork($id),
            'recordsFiltered' => $this->apiDevice->getCountOfDevicesInNetwork($id, $filter),
            'data' => $devices,
        ];

        return new JsonResponse($data);
    }

    /**
     * Return device name.
     *
     * @Route("ajax/device-name/{id}", defaults={"id": null}, options={"expose"=true}, name="app_ajax_device_name")
     *
     * @param Request $request
     *
     * @return Response
     */
    public function ajaxGetDeviceName(Request $request, $id)
    {
        //If is not ajax
        if (!$request->isXmlHttpRequest()) {
            return new Response('Bad Request', 400);
        }

        $device = $this->apiDevice->getDeviceById($id);

        if ($device) {
            $data = array('status' => 'success', 'name' => $device['name']);
        } else {
            $data = array('status' => 'error', 'msg' => 'Cannot get the device name.');
        }

        return new JsonResponse($data);
    }

    /**
     * Render Ajax page.
     *
     * @Route("ajax/device-rename", options={"expose"=true}, name="app_ajax_rename_device")
     * @Method({"POST"})
     *
     * @param Request $request
     *
     * @return Response
     */
    public function ajaxRenameDeviceAction(Request $request)
    {
        //If is not ajax
        if (!$request->isXmlHttpRequest()) {
            return new Response('Bad Request', 400);
        }

        $this->denyAccessUnlessGranted(User::ROLE_DEVICE_ADMIN);

        $data = json_decode($request->getContent(), true);
        $request->request->replace($data);

        $name = $request->request->get('name');
        $deviceId = $request->request->get('id');

        if ('' == $name) {
            $device = $this->apiDevice->getDeviceById($deviceId);

            if (!empty($device)) {
                $name = $device['name'];
            }
        }

        try {
            $this->apiDevice->setDeviceName($deviceId, $name);

            $data = array('status' => 'success', 'name' => $name);
        } catch (\Exception $e) {
            $data = array('status' => 'failed', 'name' => '');
        }

        return new JsonResponse($data);
    }

    /**
     * Return device label.
     *
     * @Route("ajax/device-label/{id}", defaults={"id": null}, options={"expose"=true}, name="app_ajax_device_label")
     *
     * @param Request $request
     *
     * @return Response
     */
    public function ajaxGetDeviceLabel(Request $request, $id)
    {
        //If is not ajax
        if (!$request->isXmlHttpRequest()) {
            return new Response('Bad Request', 400);
        }

        $device = $this->apiDevice->getDeviceById($id);

        if ($device) {
            $data = array('status' => 'success', 'label' => $device['label']);
        } else {
            $data = array('status' => 'error', 'msg' => 'Cannot get the device label.');
        }

        return new JsonResponse($data);
    }

    /**
     * Render Ajax page.
     *
     * @Route("ajax/device-relabel", options={"expose"=true}, name="app_ajax_relabel_device")
     * @Method({"POST"})
     *
     * @param Request $request
     *
     * @return Response
     */
    public function ajaxRelabelDeviceAction(Request $request)
    {
        //If is not ajax
        if (!$request->isXmlHttpRequest()) {
            return new Response('Bad Request', 400);
        }

        $this->denyAccessUnlessGranted(User::ROLE_DEVICE_ADMIN);

        $data = json_decode($request->getContent(), true);
        $request->request->replace($data);

        $label = $request->request->get('label');
        $deviceId = $request->request->get('id');

        try {
            $this->apiDevice->setDeviceLabel($deviceId, $label);

            $data = array('status' => 'success', 'label' => $label);
        } catch (\Exception $e) {
            $data = array('status' => 'failed', 'label' => '');
        }

        return new JsonResponse($data);
    }

    /**
     * Render Ajax page.
     *
     * @Route("ajax/available-devices/{id}", options={"expose"=true}, name="app_ajax_available_devices_network")
     *
     * @param Request $request
     *
     * @return Response
     */
    public function ajaxAvailableDevicesForNetworkAction(Request $request, $id)
    {
        //If is not ajax
        if (!$request->isXmlHttpRequest()) {
            return new Response('Bad Request', 400);
        }

        $devices = $this->apiDevice->getDevicesNotInNetwork($id, 0, 0, array(), array('validated' => '1'));
        //$data = array();
        $dataRouters = array();
        $dataStandAs = array();

        foreach ((array) $devices as $device) {
            //$data[$device['name']] = $device['device_id'];

            // order Standalone Clients first (ordered by id), then routers

            if (2 == $device['device_type']) {
                $dataRouters[$device['name']] = $device['device_id'];
            }
            if (1 == $device['device_type']) {
                $dataStandAs[$device['name']] = $device['device_id'];
            }
        }
        $data = array_merge($dataStandAs, $dataRouters);

        $data = ['status' => 'success', 'data' => $data];

        return new JsonResponse($data);
    }

    /**
     * Render Ajax page.
     *
     * @Route("ajax/device-network-add", options={"expose"=true}, name="app_ajax_add_device_network")
     * @Method({"POST"})
     *
     * @param Request $request
     *
     * @return Response
     */
    public function ajaxAddDeviceToNetworkAction(Request $request)
    {
        //If is not ajax
        if (!$request->isXmlHttpRequest()) {
            return new Response('Bad Request', 400);
        }

        $this->denyAccessUnlessGranted(User::ROLE_NET_MANAGER);

        $data = json_decode($request->getContent(), true);
        $request->request->replace($data);

        $networkId = $request->request->get('networkId');
        $deviceId = $request->request->get('deviceId');

        try {
            $this->apiDevice->joinDeviceNetwork($deviceId, $networkId);

            $data = array('status' => 'success', 'msg' => '');
        } catch (\Exception $e) {
            $data = array('status' => 'failed', 'msg' => '');
        }

        return new JsonResponse($data);
    }

    /**
     * Render Ajax page.
     *
     * @Route("ajax/devices-networks-add", options={"expose"=true}, name="app_ajax_add_devices_networks")
     * @Method({"POST"})
     *
     * @param Request $request
     *
     * @return Response
     */
    public function ajaxAddDevicesToNetworksAction(Request $request)
    {
        //If is not ajax
        if (!$request->isXmlHttpRequest()) {
            return new Response('Bad Request', 400);
        }

        $this->denyAccessUnlessGranted(User::ROLE_NET_MANAGER);

        $data = json_decode($request->getContent(), true);
        $request->request->replace($data);

        $networkIds = $request->request->get('networkIds');
        $deviceIds = $request->request->get('deviceIds');

        try {
            foreach ($networkIds as $networkId) {
                $this->apiDevice->joinDevicesNetwork($deviceIds, $networkId);
            }

            $data = array('status' => 'success', 'msg' => '');
        } catch (\Exception $e) {
            $data = array('status' => 'failed', 'msg' => '');
        }

        return new JsonResponse($data);
    }

    /**
     * Render Ajax page.
     *
     * @Route("ajax/device-network-leave", options={"expose"=true}, name="app_ajax_leave_device_network")
     * @Method({"POST"})
     *
     * @param Request $request
     *
     * @return Response
     */
    public function ajaxLeaveDeviceNetworkAction(Request $request)
    {
        //If is not ajax
        if (!$request->isXmlHttpRequest()) {
            return new Response('Bad Request', 400);
        }

        $this->denyAccessUnlessGranted(User::ROLE_NET_MANAGER);

        $data = json_decode($request->getContent(), true);
        $request->request->replace($data);

        $networkId = $request->request->get('networkId');
        $deviceId = $request->request->get('deviceId');

        try {
            $this->apiDevice->leaveDeviceNetwork($deviceId, $networkId);

            $data = array('status' => 'success', 'msg' => '');
        } catch (\Exception $e) {
            $data = array('status' => 'failed', 'msg' => '');
        }

        return new JsonResponse($data);
    }

    /**
     * Render Ajax page.
     *
     * @Route("ajax/device-remove", options={"expose"=true}, name="app_ajax_remove_device")
     * @Method({"POST"})
     *
     * @param Request $request
     *
     * @return Response
     */
    public function ajaxRemoveDeviceAction(Request $request)
    {
        //If is not ajax
        if (!$request->isXmlHttpRequest()) {
            return new Response('Bad Request', 400);
        }

        $this->denyAccessUnlessGranted(User::ROLE_NET_MANAGER);

        $data = json_decode($request->getContent(), true);
        $request->request->replace($data);

        $deviceId = $request->request->get('deviceId');

        if (isset($_SESSION['_deviceReturnPath'])) {
            $path = ($_SESSION['_deviceReturnPath']);
        } else {
            $router = $this->get('router');
            $path = $router->generate('app_manage_devices');
        }

        try {
            $this->apiDevice->deleteDevice($deviceId);

            $data = array('status' => 'success', 'path' => $path, 'msg' => '');
        } catch (\Exception $e) {
            $data = array('status' => 'failed', 'msg' => '');
        }

        return new JsonResponse($data);
    }

    /**
     * Render Ajax page.
     *
     * @Route("ajax/devices-remove", options={"expose"=true}, name="app_ajax_remove_devices")
     * @Method({"POST"})
     *
     * @param Request $request
     *
     * @return Response
     */
    public function ajaxRemoveDevicesAction(Request $request)
    {
        //If is not ajax
        if (!$request->isXmlHttpRequest()) {
            return new Response('Bad Request', 400);
        }

        $this->denyAccessUnlessGranted(User::ROLE_NET_MANAGER);

        $data = json_decode($request->getContent(), true);
        $request->request->replace($data);

        $ids = $request->request->get('ids');

        try {
            $this->apiDevice->deleteDevices($ids);

            $data = array('status' => 'success', 'msg' => implode(',', $ids));
        } catch (\Exception $e) {
            $data = array('status' => 'failed', 'msg' => implode(',', $ids));
        }

        return new JsonResponse($data);
    }

    /**
     * Render Ajax page.
     *
     * @Route("ajax/device-disable", options={"expose"=true}, name="app_ajax_disable_device")
     * @Method({"POST"})
     *
     * @param Request $request
     *
     * @return Response
     */
    public function ajaxDisableDeviceAction(Request $request)
    {
        //If is not ajax
        if (!$request->isXmlHttpRequest()) {
            return new Response('Bad Request', 400);
        }

        $this->denyAccessUnlessGranted(User::ROLE_DEVICE_ADMIN);

        $data = json_decode($request->getContent(), true);
        $request->request->replace($data);

        $deviceId = array($request->request->get('deviceId'));

        try {
            $this->apiDevice->disableDevices($deviceId);

            $data = array('status' => 'success', 'msg' => '');
        } catch (\Exception $e) {
            $data = array('status' => 'failed', 'msg' => '');
        }

        return new JsonResponse($data);
    }

    /**
     * Render Ajax page.
     *
     * @Route("ajax/devices-disable", options={"expose"=true}, name="app_ajax_disable_devices")
     * @Method({"POST"})
     *
     * @param Request $request
     *
     * @return Response
     */
    public function ajaxDisableDevicesAction(Request $request)
    {
        //If is not ajax
        if (!$request->isXmlHttpRequest()) {
            return new Response('Bad Request', 400);
        }

        $this->denyAccessUnlessGranted(User::ROLE_DEVICE_ADMIN);

        $data = json_decode($request->getContent(), true);
        $request->request->replace($data);

        $ids = $request->request->get('ids');

        try {
            $this->apiDevice->disableDevices($ids);

            $data = array('status' => 'success', 'msg' => implode(',', $ids));
        } catch (\Exception $e) {
            $data = array('status' => 'failed', 'msg' => implode(',', $ids));
        }

        return new JsonResponse($data);
    }

    /**
     * Render Ajax page.
     *
     * @Route("ajax/device-enable", options={"expose"=true}, name="app_ajax_enable_device")
     * @Method({"POST"})
     *
     * @param Request $request
     *
     * @return Response
     */
    public function ajaxEnableDeviceAction(Request $request)
    {
        //If is not ajax
        if (!$request->isXmlHttpRequest()) {
            return new Response('Bad Request', 400);
        }

        $this->denyAccessUnlessGranted(User::ROLE_DEVICE_ADMIN);

        $data = json_decode($request->getContent(), true);
        $request->request->replace($data);

        $deviceId = array($request->request->get('deviceId'));

        try {
            $this->apiDevice->enableDevices($deviceId);

            $data = array('status' => 'success', 'msg' => '');
        } catch (\Exception $e) {
            $data = array('status' => 'failed', 'msg' => '');
        }

        return new JsonResponse($data);
    }

    /**
     * Render Ajax page.
     *
     * @Route("ajax/devices-enable", options={"expose"=true}, name="app_ajax_enable_devices")
     * @Method({"POST"})
     *
     * @param Request $request
     *
     * @return Response
     */
    public function ajaxEnableDevicesAction(Request $request)
    {
        //If is not ajax
        if (!$request->isXmlHttpRequest()) {
            return new Response('Bad Request', 400);
        }

        $this->denyAccessUnlessGranted(User::ROLE_DEVICE_ADMIN);

        $data = json_decode($request->getContent(), true);
        $request->request->replace($data);

        $ids = $request->request->get('ids');

        try {
            $this->apiDevice->enableDevices($ids);

            $data = array('status' => 'success', 'msg' => implode(',', $ids));
        } catch (\Exception $e) {
            $data = array('status' => 'failed', 'msg' => implode(',', $ids));
        }

        return new JsonResponse($data);
    }

    /**
     * Render Ajax page.
     *
     * @Route("ajax/device-allow-internet", options={"expose"=true}, name="app_ajax_allow_internet")
     * @Method({"POST"})
     *
     * @param Request $request
     *
     * @return Response
     */
    public function ajaxAllowInternetAction(Request $request)
    {
        //If is not ajax
        if (!$request->isXmlHttpRequest()) {
            return new Response('Bad Request', 400);
        }

        $this->denyAccessUnlessGranted(User::ROLE_DEVICE_ADMIN);

        $data = json_decode($request->getContent(), true);
        $request->request->replace($data);

        $deviceId = $request->request->get('deviceId');

        try {
            $this->apiDevice->setInternetAccess($deviceId,1);

            $data = array('status' => 'success', 'msg' => '');
        } catch (\Exception $e) {
            $data = array('status' => 'failed', 'msg' => '');
        }

        return new JsonResponse($data);
    }

    /**
     * Render Ajax page.
     *
     * @Route("ajax/device-deny-internet", options={"expose"=true}, name="app_ajax_deny_internet")
     * @Method({"POST"})
     *
     * @param Request $request
     *
     * @return Response
     */
    public function ajaxDenyInternetAction(Request $request)
    {
        //If is not ajax
        if (!$request->isXmlHttpRequest()) {
            return new Response('Bad Request', 400);
        }

        $this->denyAccessUnlessGranted(User::ROLE_DEVICE_ADMIN);

        $data = json_decode($request->getContent(), true);
        $request->request->replace($data);

        $deviceId = $request->request->get('deviceId');

        try {
            $this->apiDevice->setInternetAccess($deviceId,0);

            $data = array('status' => 'success', 'msg' => '');
        } catch (\Exception $e) {
            $data = array('status' => 'failed', 'msg' => '');
        }

        return new JsonResponse($data);
    }


    /**
     * Render Ajax page.
     *
     * @Route("ajax/device-rescan", options={"expose"=true}, name="app_ajax_rescan_lans")
     * @Method({"POST"})
     *
     * @param Request $request
     *
     * @return Response
     */
    public function ajaxRescanLansAction(Request $request)
    {
        //If is not ajax
        if (!$request->isXmlHttpRequest()) {
            return new Response('Bad Request', 400);
        }

        $this->denyAccessUnlessGranted(User::ROLE_DEVICE_ADMIN);

        $data = json_decode($request->getContent(), true);
        $request->request->replace($data);

        $deviceId = $request->request->get('id');

        try {
            $this->apiDevice->rescanLans($deviceId);

            $data = array('status' => 'success');
        } catch (\Exception $e) {
            $data = array('status' => 'failed');
        }

        return new JsonResponse($data);
    }

    /**
     * Render Ajax page.
     *
     * @Route("ajax/device-mutate", options={"expose"=true}, name="app_ajax_mutate_links")
     * @Method({"POST"})
     *
     * @param Request $request
     *
     * @return Response
     */
    public function ajaxMutateLinksAction(Request $request)
    {
        //If is not ajax
        if (!$request->isXmlHttpRequest()) {
            return new Response('Bad Request', 400);
        }

        $this->denyAccessUnlessGranted(User::ROLE_DEVICE_ADMIN);

        $data = json_decode($request->getContent(), true);
        $request->request->replace($data);

        $deviceId = $request->request->get('id');

        try {
            $this->apiDevice->mutateLinks($deviceId);

            $data = array('status' => 'success');
        } catch (\Exception $e) {
            $data = array('status' => 'failed');
        }

        return new JsonResponse($data);
    }

    /*
    /**
     * Render Ajax page.
     *
     * @Route("ajax/device-create-link", options={"expose"=true}, name="app_ajax_create_link")
     * @Method({"POST"})
     *
     * @param Request $request
     *
     * @return Response
     */

    /*    public function ajaxCreateLinkAction(Request $request){
    
            //If is not ajax
            if(!$request->isXmlHttpRequest()){
    
                return new Response('Bad Request', 400);
    
            }
    
            $this->denyAccessUnlessGranted(User::ROLE_DEVICE_ADMIN);
    
            $data = json_decode($request->getContent(), true);
            $request->request->replace($data);
    
            $deviceId = $request->request->get('id');
            $ip = $request->request->get('ip');
    
    
            try{
    
                $link = $ip;
    
    
                $data = array('status' => 'success', 'link' => $link);
    
                var_dump($data);
    
            }catch (\Exception $e){
    
                $data = array('status' => 'failed');
    
            }
    
            return new JsonResponse($data);
    
        }
    */

    /**
     * @param $request
     * @param array $columns
     *
     * @return array
     */
    private function parseData($request, $columns = self::ROUTER_TABLE_COLUMNS)
    {
        $start = $request->query->getInt('start');
        $length = $request->query->getInt('length');

        // Individual column filtering
        $filterColumns = $request->query->get('columns');
        $search = $request->query->get('search');

        $filter = array();

        // probably search box that searches only in name of router
        if (isset($search['value']) && !empty($search['value'])) {
            $filter[$columns[2]] = '%'.$search['value'].'%';
        } else {
            //individual columns in footer search
            foreach ($filterColumns as $key => $column) {
                $columnIdx = array_search($column['data'], $columns);
                //column name

                if ($columnIdx && '' != $column['search']['value']) {
                    /*
                    if(sloupecek = connected) {
                        pokud tyto hodnoty vybrané search:
            $filter['disabled'] = '1';
            disable: where disabled == 1
            online: where online = 1
                disabled: where online = 0 and disabled = 0
        }
                    */
                    //if($column[$key] == "")

                    //if($columnIdx == 'online'){
                    //if($column['search']['value'] == "2"){
                    //    $filter[$columns[ $columnIdx ]] = "";
                    //}
                    //  $length = "1";

                    //} else {

                    $filter[$columns[$columnIdx]] = '%'.$column['search']['value'].'%';

                    //}
                }
            }
        }

        $order = $request->query->get('order');
        $aOrder = array();

        foreach ($order as $item) {
            $aOrder[$columns[$item['column']]] = $item['dir'];
        }

        return array($start, $length, $aOrder, $filter);
    }

    /**
     * @param $devices
     *
     * @return array
     */
    private function getDisabled($devices)
    {
        //get disabled routers as online with value "2" (in column "connected")
        foreach ($devices as &$device) {
            if ('1' == $device['disabled']) {
                $device['online'] = '2';
            }
        }

        return $devices;
    }

    /**
     * @param $filter
     *
     * @return array
     */
    private function filterDisabled($filter)
    {
        //change filter request for nonexisting value 2 in column connected
        if (array_key_exists('online', $filter)) {
            //throw new \Exception("text: " . $filter["online"]);
            // vypisuje do logu i pri ajax erroru !!!
            if ('%2%' == $filter['online']) {
                $filter['online'] = '0';
                $filter['disabled'] = '1';
            } else {
                $filter['disabled'] = '0';
            }
        }

        /*
       if($filter["online"] == "2"){
           $filter["disabled"] = "1";
           $filter["online"] = "0";
       }
       */
        //doplnit dalsi veci do filtru if sloupecek, ... poslední
        /*
        if(sloupecek = connected) {
            pokud tyto hodnoty vybrané search:
            $filter['disabled'] = '1';
            disable: where disabled == 1
            online: where online = 1
                disabled: where online = 0 and disabled = 0
        }
        */

        return $filter;
    }

    /**
     * @param $order
     *
     * @return array
     */
    private function orderDisabled($order)
    {
        //change order request for online column so it orders also disabled devices
        if (array_key_exists('online', $order)) {
            if ('asc' == $order['online']) {
                $order['disabled'] = 'asc';
                $order['online'] = 'desc';
            } else {
                $order['disabled'] = 'desc';
                $order['online'] = 'asc';
            }
        }
        /*
        razeni: nejdriv podle disabled, pak podle online
        */
        /*
        //spravne razeni sloupecku connected - podle dvou, v parse data
        $aOrder = array();
        $aOrder["disabled"] = "asc";
        $aOrder["online"] = "desc";
        */
        return $order;
    }

    /**
     * Save return path.
     *
     * @Route("ajax/device-save-return", options={"expose"=true}, name="app_ajax_save_return")
     * @Method({"POST"})
     *
     * @param Request $request
     *
     * @return Response
     */
    public function ajaxSaveReturnAction(Request $request)
    {
        //If is not ajax
        if (!$request->isXmlHttpRequest()) {
            return new Response('Bad Request', 400);
        }

        //$this->denyAccessUnlessGranted(User::ROLE_NET_MANAGER);

        $data = json_decode($request->getContent(), true);
        $request->request->replace($data);

        $path = $request->request->get('path');

        try {
            $_SESSION['_deviceReturnPath'] = $path;

            $data = array('status' => 'success', 'msg' => '');
        } catch (\Exception $e) {
            $data = array('status' => 'failed', 'msg' => '');
        }

        return new JsonResponse($data);
    }

    /**
     * @param $countryCode
     *
     * @return string
     */
    /*
    private function getCountry($countryCode) {
        $response = $this->forward('AppBundle\Controller\DevicesController::getCountry', ['countryCode' => $countryCode]);
        return $response;
    }
    */
    private function getCountry($countryCode)
    {
        switch ($countryCode) {
            case 'AD': return 'Andorra'; break;
            case 'AE': return 'United Arab Emirates'; break;
            case 'AF': return 'Afghanistan'; break;
            case 'AG': return 'Antigua and Barbuda'; break;
            case 'AI': return 'Anguilla'; break;
            case 'AL': return 'Albania'; break;
            case 'AM': return 'Armenia'; break;
            case 'AN': return 'Netherlands Antilles'; break;
            case 'AO': return 'Angola'; break;
            case 'AQ': return 'Antarctica'; break;
            case 'AR': return 'Argentina'; break;
            case 'AS': return 'American Samoa'; break;
            case 'AT': return 'Austria'; break;
            case 'AU': return 'Australia'; break;
            case 'AW': return 'Aruba'; break;
            case 'AX': return 'Åland Islands'; break;
            case 'AZ': return 'Azerbaijan'; break;
            case 'BA': return 'Bosnia and Herzegovina'; break;
            case 'BB': return 'Barbados'; break;
            case 'BD': return 'Bangladesh'; break;
            case 'BE': return 'Belgium'; break;
            case 'BF': return 'Burkina Faso'; break;
            case 'BG': return 'Bulgaria'; break;
            case 'BH': return 'Bahrain'; break;
            case 'BI': return 'Burundi'; break;
            case 'BJ': return 'Benin'; break;
            case 'BL': return 'Saint Barthélemy'; break;
            case 'BM': return 'Bermuda'; break;
            case 'BN': return 'Brunei'; break;
            case 'BO': return 'Bolivia'; break;
            case 'BQ': return 'British Antarctic Territory'; break;
            case 'BR': return 'Brazil'; break;
            case 'BS': return 'Bahamas'; break;
            case 'BT': return 'Bhutan'; break;
            case 'BV': return 'Bouvet Island'; break;
            case 'BW': return 'Botswana'; break;
            case 'BY': return 'Belarus'; break;
            case 'BZ': return 'Belize'; break;
            case 'CA': return 'Canada'; break;
            case 'CC': return 'Cocos [Keeling] Islands'; break;
            case 'CD': return 'Congo - Kinshasa'; break;
            case 'CF': return 'Central African Republic'; break;
            case 'CG': return 'Congo - Brazzaville'; break;
            case 'CH': return 'Switzerland'; break;
            case 'CI': return 'Côte d’Ivoire'; break;
            case 'CK': return 'Cook Islands'; break;
            case 'CL': return 'Chile'; break;
            case 'CM': return 'Cameroon'; break;
            case 'CN': return 'China'; break;
            case 'CO': return 'Colombia'; break;
            case 'CR': return 'Costa Rica'; break;
            case 'CS': return 'Serbia and Montenegro'; break;
            case 'CT': return 'Canton and Enderbury Islands'; break;
            case 'CU': return 'Cuba'; break;
            case 'CV': return 'Cape Verde'; break;
            case 'CX': return 'Christmas Island'; break;
            case 'CY': return 'Cyprus'; break;
            case 'CZ': return 'Czech Republic'; break;
            case 'DD': return 'East Germany'; break;
            case 'DE': return 'Germany'; break;
            case 'DJ': return 'Djibouti'; break;
            case 'DK': return 'Denmark'; break;
            case 'DM': return 'Dominica'; break;
            case 'DO': return 'Dominican Republic'; break;
            case 'DZ': return 'Algeria'; break;
            case 'EC': return 'Ecuador'; break;
            case 'EE': return 'Estonia'; break;
            case 'EG': return 'Egypt'; break;
            case 'EH': return 'Western Sahara'; break;
            case 'ER': return 'Eritrea'; break;
            case 'ES': return 'Spain'; break;
            case 'ET': return 'Ethiopia'; break;
            case 'FI': return 'Finland'; break;
            case 'FJ': return 'Fiji'; break;
            case 'FK': return 'Falkland Islands'; break;
            case 'FM': return 'Micronesia'; break;
            case 'FO': return 'Faroe Islands'; break;
            case 'FQ': return 'French Southern and Antarctic Territories'; break;
            case 'FR': return 'France'; break;
            case 'FX': return 'Metropolitan France'; break;
            case 'GA': return 'Gabon'; break;
            case 'GB': return 'United Kingdom'; break;
            case 'GD': return 'Grenada'; break;
            case 'GE': return 'Georgia'; break;
            case 'GF': return 'French Guiana'; break;
            case 'GG': return 'Guernsey'; break;
            case 'GH': return 'Ghana'; break;
            case 'GI': return 'Gibraltar'; break;
            case 'GL': return 'Greenland'; break;
            case 'GM': return 'Gambia'; break;
            case 'GN': return 'Guinea'; break;
            case 'GP': return 'Guadeloupe'; break;
            case 'GQ': return 'Equatorial Guinea'; break;
            case 'GR': return 'Greece'; break;
            case 'GS': return 'South Georgia and the South Sandwich Islands'; break;
            case 'GT': return 'Guatemala'; break;
            case 'GU': return 'Guam'; break;
            case 'GW': return 'Guinea-Bissau'; break;
            case 'GY': return 'Guyana'; break;
            case 'HK': return 'Hong Kong SAR China'; break;
            case 'HM': return 'Heard Island and McDonald Islands'; break;
            case 'HN': return 'Honduras'; break;
            case 'HR': return 'Croatia'; break;
            case 'HT': return 'Haiti'; break;
            case 'HU': return 'Hungary'; break;
            case 'ID': return 'Indonesia'; break;
            case 'IE': return 'Ireland'; break;
            case 'IL': return 'Israel'; break;
            case 'IM': return 'Isle of Man'; break;
            case 'IN': return 'India'; break;
            case 'IO': return 'British Indian Ocean Territory'; break;
            case 'IQ': return 'Iraq'; break;
            case 'IR': return 'Iran'; break;
            case 'IS': return 'Iceland'; break;
            case 'IT': return 'Italy'; break;
            case 'JE': return 'Jersey'; break;
            case 'JM': return 'Jamaica'; break;
            case 'JO': return 'Jordan'; break;
            case 'JP': return 'Japan'; break;
            case 'JT': return 'Johnston Island'; break;
            case 'KE': return 'Kenya'; break;
            case 'KG': return 'Kyrgyzstan'; break;
            case 'KH': return 'Cambodia'; break;
            case 'KI': return 'Kiribati'; break;
            case 'KM': return 'Comoros'; break;
            case 'KN': return 'Saint Kitts and Nevis'; break;
            case 'KP': return 'North Korea'; break;
            case 'KR': return 'South Korea'; break;
            case 'KW': return 'Kuwait'; break;
            case 'KY': return 'Cayman Islands'; break;
            case 'KZ': return 'Kazakhstan'; break;
            case 'LA': return 'Laos'; break;
            case 'LB': return 'Lebanon'; break;
            case 'LC': return 'Saint Lucia'; break;
            case 'LI': return 'Liechtenstein'; break;
            case 'LK': return 'Sri Lanka'; break;
            case 'LR': return 'Liberia'; break;
            case 'LS': return 'Lesotho'; break;
            case 'LT': return 'Lithuania'; break;
            case 'LU': return 'Luxembourg'; break;
            case 'LV': return 'Latvia'; break;
            case 'LY': return 'Libya'; break;
            case 'MA': return 'Morocco'; break;
            case 'MC': return 'Monaco'; break;
            case 'MD': return 'Moldova'; break;
            case 'ME': return 'Montenegro'; break;
            case 'MF': return 'Saint Martin'; break;
            case 'MG': return 'Madagascar'; break;
            case 'MH': return 'Marshall Islands'; break;
            case 'MI': return 'Midway Islands'; break;
            case 'MK': return 'Macedonia'; break;
            case 'ML': return 'Mali'; break;
            case 'MM': return 'Myanmar [Burma]'; break;
            case 'MN': return 'Mongolia'; break;
            case 'MO': return 'Macau SAR China'; break;
            case 'MP': return 'Northern Mariana Islands'; break;
            case 'MQ': return 'Martinique'; break;
            case 'MR': return 'Mauritania'; break;
            case 'MS': return 'Montserrat'; break;
            case 'MT': return 'Malta'; break;
            case 'MU': return 'Mauritius'; break;
            case 'MV': return 'Maldives'; break;
            case 'MW': return 'Malawi'; break;
            case 'MX': return 'Mexico'; break;
            case 'MY': return 'Malaysia'; break;
            case 'MZ': return 'Mozambique'; break;
            case 'NA': return 'Namibia'; break;
            case 'NC': return 'New Caledonia'; break;
            case 'NE': return 'Niger'; break;
            case 'NF': return 'Norfolk Island'; break;
            case 'NG': return 'Nigeria'; break;
            case 'NI': return 'Nicaragua'; break;
            case 'NL': return 'Netherlands'; break;
            case 'NO': return 'Norway'; break;
            case 'NP': return 'Nepal'; break;
            case 'NQ': return 'Dronning Maud Land'; break;
            case 'NR': return 'Nauru'; break;
            case 'NT': return 'Neutral Zone'; break;
            case 'NU': return 'Niue'; break;
            case 'NZ': return 'New Zealand'; break;
            case 'OM': return 'Oman'; break;
            case 'PA': return 'Panama'; break;
            case 'PC': return 'Pacific Islands Trust Territory'; break;
            case 'PE': return 'Peru'; break;
            case 'PF': return 'French Polynesia'; break;
            case 'PG': return 'Papua New Guinea'; break;
            case 'PH': return 'Philippines'; break;
            case 'PK': return 'Pakistan'; break;
            case 'PL': return 'Poland'; break;
            case 'PM': return 'Saint Pierre and Miquelon'; break;
            case 'PN': return 'Pitcairn Islands'; break;
            case 'PR': return 'Puerto Rico'; break;
            case 'PS': return 'Palestinian Territories'; break;
            case 'PT': return 'Portugal'; break;
            case 'PU': return 'U.S. Miscellaneous Pacific Islands'; break;
            case 'PW': return 'Palau'; break;
            case 'PY': return 'Paraguay'; break;
            case 'PZ': return 'Panama Canal Zone'; break;
            case 'QA': return 'Qatar'; break;
            case 'RE': return 'Réunion'; break;
            case 'RO': return 'Romania'; break;
            case 'RS': return 'Serbia'; break;
            case 'RU': return 'Russia'; break;
            case 'RW': return 'Rwanda'; break;
            case 'SA': return 'Saudi Arabia'; break;
            case 'SB': return 'Solomon Islands'; break;
            case 'SC': return 'Seychelles'; break;
            case 'SD': return 'Sudan'; break;
            case 'SE': return 'Sweden'; break;
            case 'SG': return 'Singapore'; break;
            case 'SH': return 'Saint Helena'; break;
            case 'SI': return 'Slovenia'; break;
            case 'SJ': return 'Svalbard and Jan Mayen'; break;
            case 'SK': return 'Slovakia'; break;
            case 'SL': return 'Sierra Leone'; break;
            case 'SM': return 'San Marino'; break;
            case 'SN': return 'Senegal'; break;
            case 'SO': return 'Somalia'; break;
            case 'SR': return 'Suriname'; break;
            case 'ST': return 'São Tomé and Príncipe'; break;
            case 'SU': return 'Union of Soviet Socialist Republics'; break;
            case 'SV': return 'El Salvador'; break;
            case 'SY': return 'Syria'; break;
            case 'SZ': return 'Swaziland'; break;
            case 'TC': return 'Turks and Caicos Islands'; break;
            case 'TD': return 'Chad'; break;
            case 'TF': return 'French Southern Territories'; break;
            case 'TG': return 'Togo'; break;
            case 'TH': return 'Thailand'; break;
            case 'TJ': return 'Tajikistan'; break;
            case 'TK': return 'Tokelau'; break;
            case 'TL': return 'Timor-Leste'; break;
            case 'TM': return 'Turkmenistan'; break;
            case 'TN': return 'Tunisia'; break;
            case 'TO': return 'Tonga'; break;
            case 'TR': return 'Turkey'; break;
            case 'TT': return 'Trinidad and Tobago'; break;
            case 'TV': return 'Tuvalu'; break;
            case 'TW': return 'Taiwan'; break;
            case 'TZ': return 'Tanzania'; break;
            case 'UA': return 'Ukraine'; break;
            case 'UG': return 'Uganda'; break;
            case 'UM': return 'U.S. Minor Outlying Islands'; break;
            case 'US': return 'United States'; break;
            case 'UY': return 'Uruguay'; break;
            case 'UZ': return 'Uzbekistan'; break;
            case 'VA': return 'Vatican City'; break;
            case 'VC': return 'Saint Vincent and the Grenadines'; break;
            case 'VD': return 'North Vietnam'; break;
            case 'VE': return 'Venezuela'; break;
            case 'VG': return 'British Virgin Islands'; break;
            case 'VI': return 'U.S. Virgin Islands'; break;
            case 'VN': return 'Vietnam'; break;
            case 'VU': return 'Vanuatu'; break;
            case 'WF': return 'Wallis and Futuna'; break;
            case 'WK': return 'Wake Island'; break;
            case 'WS': return 'Samoa'; break;
            case 'YD': return "People's Democratic Republic of Yemen"; break;
            case 'YE': return 'Yemen'; break;
            case 'YT': return 'Mayotte'; break;
            case 'ZA': return 'South Africa'; break;
            case 'ZM': return 'Zambia'; break;
            case 'ZW': return 'Zimbabwe'; break;
            case 'ZZ': return 'Unknown or Invalid Region'; break;
        }
    }
}
