<?php

namespace AppBundle\Controller\Ajax;

use AppBundle\Entity\User;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Route;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Method;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpFoundation\JsonResponse;

/**
 * GentelellaController.
 */
class AjaxPrevalidationController extends AjaxBaseController
{

    /**
     * Render Ajax page.
     *
     * @Route("ajax/prevalidation-table", name="app_ajax_prevalidation_table")
     *
     * @param Request $request
     *
     * @return Response
     */
    public function ajaxAction(Request $request){

        //If is not ajax
        if(!$request->isXmlHttpRequest()){

            return new Response('Bad Request', 400);

        }

        list($start, $length, $aOrder, $filter) = $this->parseData($request);


        $prevalidationList = $this->apiDevice->getPrevalidationList($start, $length, $aOrder, $filter);


        if(empty($prevalidationList)){

            $prevalidationList = array();

        }

        $data =  [

            "recordsTotal" => $this->apiDevice->getCountOfPrevalidated(),
            "recordsFiltered" => $this->apiDevice->getCountOfPrevalidated($filter),
            "data" => $prevalidationList

        ];

        return new JsonResponse($data);


    }



    /**
     * @param $request
     * @return array
     */
    private function parseData($request){

        $parametersToValidate = $request->query->all();

        $start = $request->query->getInt('start');
        $length = $request->query->getInt('length');

        //$search = $request->query->get('search');

        $columns = array(

            0 => 'udid',
            1 => 'serial',
            2 => 'mac',
            3 => 'imei',
            4 => 'name',

        );


        // Individual column filtering
        $filterColumns = $request->query->get('columns');
        $search = $request->query->get('search');

        $filter = array();


        if(isset($search['value']) && !empty($search['value'])){

            $filter[$columns[ 1 ]] = '%' . $search['value'] . '%';

        }else{

            foreach ($filterColumns as $key => $column){

                $columnIdx = array_search( $column['data'], $columns );

                if($columnIdx && $column['search']['value'] != ''){

                    $filter[$columns[ $columnIdx ]] = '%' . $column['search']['value'] . '%';

                }

            }

        }



        $order = $request->query->get('order');
        $aOrder = array();

        foreach ($order as $item){

            $aOrder[$columns[$item['column']]] = $item['dir'];
        }

        return array($start, $length, $aOrder, $filter);

    }

}
