<?php

namespace AppBundle\Controller;

use AppBundle\Api\CsServerSettings;
use AppBundle\Entity\User;
use AppBundle\Form\AppUpdateFormType;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Route;
use Symfony\Bundle\FrameworkBundle\Controller\Controller;
use Symfony\Component\Form\Util\ServerParams;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;

/**
 * Class AppManagementController
 * @package AppBundle\Controller
 */
class AppManagementController extends Controller
{

    /**
     * @Route("/application-management", name="app_management")
     *
     * @param Request $request
     *
     * @return Response
     */
    public function indexAction(Request $request){

        $this->denyAccessUnlessGranted(User::ROLE_SYSTEM_ADMIN);

        $upgradeAppForm = $this->createForm(AppUpdateFormType::class, null, array(

            'action' => $this->generateUrl('app_upgrade'),
            'method' => 'POST'

        ));

        $uploadLicenseForm = $this->createForm(AppUpdateFormType::class, null, array(

            'action' => $this->generateUrl('app_add_license'),
            'method' => 'POST'

        ));

        /** @var CsServerSettings $csServerSettings */
        $csServerSettings = $this->container->get('cs_api_server_settings');

        return $this->render('app/pages/appManagement/index.html.twig', [

            'licenseInformations' => $csServerSettings->getLicenseInfo(),
            'upgradeAppForm'      => $upgradeAppForm->createView(),
            'uploadLicenseForm'   => $uploadLicenseForm->createView(),
            'statusCsDaemon'      => $csServerSettings->getServiceStatus($csServerSettings::SERVICE_CS_DAEMON),
            'statusDsDaemon'      => $csServerSettings->getServiceStatus($csServerSettings::SERVICE_DS_DAEMON),
            'statusRoadwarrior'   => $csServerSettings->getServiceStatus($csServerSettings::SERVICE_ROADWARRIOR),

        ]);

    }

    /**
     * @Route("/application-upgrade", name="app_upgrade")
     *
     * @param Request $request
     *
     * @return Response
     */
    public function appUpgradeAction(Request $request){

        $this->denyAccessUnlessGranted(User::ROLE_SYSTEM_ADMIN);

        $form = $this->createForm(AppUpdateFormType::class, null, array(

            'action' => $this->generateUrl('app_upgrade'),
            'method' => 'POST'

        ));

        try{
            $form->handleRequest($request);
        } catch (Exception $e){
            $this->addFlash(
            'error',
            'File upload failed.'
            );
            return $this->redirect($this->generateUrl('login'));
        }

        if ($form->isSubmitted() && $form->isValid()) {
            $file = $form->get('file')->getData();
            $fileName = $file->getClientOriginalName();

            // Move the file to the directory where brochures are stored
            try {
                $file->move(
                    $this->getParameter('upload_directory'),
                    $fileName
                );
            } catch (FileException $e) {
                $this->addFlash(
                    'error',
                    'File upload failed.'
                );
                return $this->redirect($this->generateUrl('app_management'));
            }

            /** @var CsServerSettings $csServerSettings */
            $csServerSettings = $this->container->get('cs_api_server_settings');

            //$result = $csServerSettings->uploadFile($this->getParameter('upload_directory').'/'.$fileName);

            /*
             * // puvodni
             *
            try{

                $result = $csServerSettings->uploadFile($this->getParameter('upload_directory').'/'.$fileName);

            }catch (\Exception $e){

                $errorMessage = $e->getMessage();

            }
            */


            $result = $csServerSettings->uploadFile($this->getParameter('upload_directory').'/'.$fileName);


            if ($result) {

                $this->addFlash(
                    'error',
                    $result
                );

            } else {

                $this->addFlash(
                    'notice',
                    'File was successfully uploaded, upgrade in progress...'
                );
		            session_destroy();

            }





/*
            if(isset($errorMessage)){

                $this->addFlash(
                    'fwError',
                    $errorMessage . ' '
                );
            }else{

                $this->addFlash(
                    'notice',
                    'File was successfully uploaded, upgrade in progress...'
                );

            }
*/




/*
 * //puvodni
 *
            if($result){

                $this->addFlash(
                    'notice',
                    'File was successfully uploaded, upgrade in progress...'
                );

            }else{

                if(isset($errorMessage)){

                    $this->addFlash(
                        'error',
                        $errorMessage
                    );
                }else{
                    $this->addFlash(
                        'error',
                        'There was an error during the upgrade.'
                    );
                };
            }
*/


            return $this->redirect($this->generateUrl('app_management'));

        }

    }

    /**
     * @Route("/application-add-license", name="app_add_license")
     *
     * @param Request $request
     *
     * @return Response
     */
    public function addLicenceAction(Request $request){

        $this->denyAccessUnlessGranted(User::ROLE_SYSTEM_ADMIN);

        $form = $this->createForm(AppUpdateFormType::class, null, array(

            'action' => $this->generateUrl('app_add_license'),
            'method' => 'POST'

        ));
        $form->handleRequest($request);

        if ($form->isSubmitted() && $form->isValid()) {

            $file = $form->get('file')->getData();

            $fileName = 'uploadedfile';

            // Move the file to the directory where brochures are stored
            try {
                $file->move(
                    $this->getParameter('upload_directory'),
                    $fileName
                );
            } catch (FileException $e) {

                $this->addFlash(
                    'error',
                    'File upload failed!'
                );

                return $this->redirect($this->generateUrl('app_management'));
            }

            /** @var CsServerSettings $csServerSettings */
            $csServerSettings = $this->container->get('cs_api_server_settings');

            try{

                $csServerSettings->addLicence($this->getParameter('upload_directory').'/'.$fileName);

                $this->addFlash(
                    'notice',
                    'License was successfully uploaded!'
                );

            }catch (\Exception $e){

                $this->addFlash(
                    'error',
                    $e->getMessage()
                );

            }

            return $this->redirect($this->generateUrl('app_management'));

        }

    }

}
