<?php

namespace AppBundle\Form;

use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\OptionsResolver\OptionsResolver;
use Symfony\Component\Validator\Constraints as Assert;
use Symfony\Component\Form\Extension\Core\Type;
use Symfony\Component\Form\FormView;
use Symfony\Component\Form\FormInterface;

class DeviceOneToOneNat extends AbstractType
{

    /**
     * @var array
     */
    private $oneToOneNat;

    /**
     * @var array
     */
    private $oneToOneNatPending;

    /**
     * @param FormBuilderInterface $builder
     * @param array $options
     */
    public function buildForm(FormBuilderInterface $builder, array $options){

        $builder->add('local_ip',Type\TextType::class, [
            'label' => 'Email',
            'constraints' =>[
                new Assert\NotBlank([
                    'message' => 'This field can not be blank'
                ]),
                new Assert\Ip([
                    'message' => 'Invalid IP address'
                ])
            ]]);
        $builder->add('local_mask',Type\TextType::class, ['label' => 'Email',
            'constraints' =>[
            new Assert\NotBlank([
                'message' => 'This field can not be blank'
            ]),
            new Assert\Ip([
                'message' => 'Invalid IP address'
            ])
        ]]);
        $builder->add('virtual_ip',Type\TextType::class, ['label' => 'Email',
            'constraints' =>[
                new Assert\NotBlank([
                    'message' => 'This field can not be blank'
                ]),
                new Assert\Ip([
                    'message' => 'Invalid IP address'
                ])
            ]]);
        $builder->add('virtual_mask',Type\TextType::class, ['label' => 'Email',
            'constraints' =>[
                new Assert\NotBlank([
                    'message' => 'This field can not be blank'
                ]),
                new Assert\Ip([
                    'message' => 'Invalid IP address'
                ])
            ]]);

        $builder->add('o2o_nat_id', Type\HiddenType::class);
    }

    /**
     * @param FormView $view
     * @param FormInterface $form
     * @param array $options
     */
    public function buildView(FormView $view, FormInterface $form, array $options){

        $view->vars = array_replace($view->vars, array(
            'oneToOneNat' => $options['oneToOneNat'],
            'oneToOneNatPending' => array(), //$options['oneToOneNatPending'],
        ));

    }

    /**
     * @param OptionsResolver $resolver
     */
    public function configureOptions(OptionsResolver $resolver){

        $resolver->setDefaults(array(
            //'data_class' => Network::class,
        ));

        $resolver->setDefaults(array(
            'oneToOneNat'  => $this->oneToOneNat,
            'oneToOneNatPending'  => $this->oneToOneNatPending,
        ));

        $resolver->setRequired('oneToOneNat');
        $resolver->setRequired('oneToOneNatPending');

    }

    /**
     * @return null|string
     */
    public function getBlockPrefix(){

        return 'OneToOneNatType';

    }

}