<?php

namespace AppBundle\Twig\Extension;

use Symfony\Component\Security\Core\Authentication\Token\Storage\TokenStorageInterface;
use Symfony\Component\Security\Core\Authorization\AuthorizationCheckerInterface;
use Symfony\Component\Security\Core\Authorization\AccessDecisionManagerInterface;
use Symfony\Component\Security\Http\AccessMap;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\Routing\Router;
use Twig\Extension\AbstractExtension;

class SecurityTwigExtension extends AbstractExtension
{
    private $context;
    private $tokenStorage;
    private $accessDecisionManager;
    private $map;
    private $router;

    public function __construct(AuthorizationCheckerInterface $context,
                                TokenStorageInterface $tokenStorage,
                                AccessDecisionManagerInterface $accessDecisionManager, AccessMap $map,
                                Router $router)
    {
        $this->context = $context;
        $this->tokenStorage = $tokenStorage;
        $this->accessDecisionManager = $accessDecisionManager;
        $this->map = $map;
        $this->router = $router;
    }

    /**
     * {@inheritdoc}
     */
    public function getFunctions()
    {
        return array(
            'is_accessible' => new \Twig_SimpleFunction('isAccesible', array($this, 'isAccesible')),
        );
    }

    public function isAccesible($path)
    {
        if (null === $token = $this->tokenStorage->getToken()) {
            return true;
        }

        $baseUrl = $this->router->getContext()->getBaseUrl();
        $path = substr($path, strlen($baseUrl));

        $request = Request::create($path);

        list($attributes, $channel) = $this->map->getPatterns($request);

        if (null === $attributes) {
            return true;
        }

        if (!$token->isAuthenticated()) {
            return false;
        }

        return $this->accessDecisionManager->decide($token,
            $attributes, $request);
    }

    public function getName()
    {
        return 'is_accessible';
    }

} 